Package: planttfhunter
Title: Identification and classification of plant transcription factors
Version: 1.12.0
Date: 2022-03-10
Authors@R: 
    c(
    person(given = "Fabrício",
           family = "Almeida-Silva",
           role = c("aut", "cre"),
           email = "fabricio_almeidasilva@hotmail.com",
           comment = c(ORCID = "0000-0002-5314-2964")),
    person(given = "Yves",
           family = "Van de Peer",
           role = "aut",
           email = "yves.vandepeer@psb.vib-ugent.be",
           comment = c(ORCID = "0000-0003-4327-3730"))
    )
Description: planttfhunter is used to identify plant transcription factors (TFs) 
    from protein sequence data and classify them into families and subfamilies
    using the classification scheme implemented in PlantTFDB.
    TFs are identified using pre-built hidden Markov model profiles
    for DNA-binding domains. Then, auxiliary and forbidden domains are
    used with DNA-binding domains to classify TFs into families and
    subfamilies (when applicable). Currently, TFs can be classified in
    58 different TF families/subfamilies.
License: GPL-3
URL: https://github.com/almeidasilvaf/planttfhunter
BugReports: https://support.bioconductor.org/t/planttfhunter
biocViews: Software, Transcription, FunctionalPrediction,
        GenomeAnnotation, FunctionalGenomics, HiddenMarkovModel,
        Sequencing, Classification
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.1
SystemRequirements: HMMER <http://hmmer.org/>
Imports: Biostrings, SummarizedExperiment, utils, methods
Suggests: BiocStyle, covr, sessioninfo, knitr, rmarkdown, testthat (>=
        3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
Depends: R (>= 4.2.0)
LazyData: false
git_url: https://git.bioconductor.org/packages/planttfhunter
git_branch: RELEASE_3_23
git_last_commit: 677b1c7
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:08:02 UTC; biocbuild
Author: Fabrício Almeida-Silva [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5314-2964>),
  Yves Van de Peer [aut] (ORCID: <https://orcid.org/0000-0003-4327-3730>)
Maintainer: Fabrício Almeida-Silva <fabricio_almeidasilva@hotmail.com>
Built: R 4.6.0; ; 2026-04-29 04:54:44 UTC; unix

# Generated by roxygen2: do not edit by hand

export(annotate_pfam)
export(classify_tfs)
export(get_tf_counts)
export(hmmer_is_installed)
importFrom(Biostrings,writeXStringSet)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(methods,is)
importFrom(utils,read.csv)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# planttfhunter <img src="man/figures/logo.png" align="right" height="139" />

<!-- badges: start -->

[![GitHub
issues](https://img.shields.io/github/issues/almeidasilvaf/planttfhunter)](https://github.com/almeidasilvaf/planttfhunter/issues)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![R-CMD-check-bioc](https://github.com/almeidasilvaf/planttfhunter/workflows/R-CMD-check-bioc/badge.svg)](https://github.com/almeidasilvaf/planttfhunter/actions)
[![Codecov test
coverage](https://codecov.io/gh/almeidasilvaf/planttfhunter/branch/devel/graph/badge.svg)](https://codecov.io/gh/almeidasilvaf/planttfhunter?branch=devel)
<!-- badges: end -->

The goal of **planttfhunter** is to identify plant transcription factors
from protein sequence data and classify them into families and
subfamilies using the classification scheme implemented in
[PlantTFDB](https://doi.org/10.1093/nar/gkz1020).

## Installation instructions

Get the latest stable `R` release from
[CRAN](http://cran.r-project.org/). Then install **planttfhunter** from
[Bioconductor](http://bioconductor.org/) using the following code:

``` r
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("planttfhunter")
```

And the development version from
[GitHub](https://github.com/almeidasilvaf/planttfhunter) with:

``` r
BiocManager::install("almeidasilvaf/planttfhunter")
```

## Citation

Below is the citation output from using `citation('planttfhunter')` in
R. Please run this yourself to check for any updates on how to cite
**planttfhunter**.

``` r
print(citation('planttfhunter'), bibtex = TRUE)
#> 
#> To cite package 'planttfhunter' in publications use:
#> 
#>   Almeida-Silva F, Van de Peer Y (2022). _planttfhunter: Identification
#>   and classification of plant transcription factors_. R package version
#>   0.99.0, <https://github.com/almeidasilvaf/planttfhunter>.
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Manual{,
#>     title = {planttfhunter: Identification and classification of plant transcription factors},
#>     author = {Fabrício Almeida-Silva and Yves {Van de Peer}},
#>     year = {2022},
#>     note = {R package version 0.99.0},
#>     url = {https://github.com/almeidasilvaf/planttfhunter},
#>   }
```

Please note that the **planttfhunter** project was only made possible
thanks to many other R and bioinformatics software authors, which are
cited either in the vignettes and/or the paper(s) describing this
package.

## Code of Conduct

Please note that the **planttfhunter** project is released with a
[Contributor Code of
Conduct](http://bioconductor.org/about/code-of-conduct/). By
contributing to this project, you agree to abide by its terms.

## Development tools

- Continuous code testing is possible thanks to [GitHub
  actions](https://www.tidyverse.org/blog/2020/04/usethis-1-6-0/)
  through *[usethis](https://CRAN.R-project.org/package=usethis)*,
  *[remotes](https://CRAN.R-project.org/package=remotes)*, and
  *[rcmdcheck](https://CRAN.R-project.org/package=rcmdcheck)* customized
  to use [Bioconductor’s docker
  containers](https://www.bioconductor.org/help/docker/) and
  *[BiocCheck](https://bioconductor.org/packages/3.15/BiocCheck)*.
- Code coverage assessment is possible thanks to
  [codecov](https://codecov.io/gh) and
  *[covr](https://CRAN.R-project.org/package=covr)*.
- The [documentation
  website](http://almeidasilvaf.github.io/planttfhunter) is
  automatically updated thanks to
  *[pkgdown](https://CRAN.R-project.org/package=pkgdown)*.
- The documentation is formatted thanks to
  *[devtools](https://CRAN.R-project.org/package=devtools)* and
  *[roxygen2](https://CRAN.R-project.org/package=roxygen2)*.

This package was developed using
*[biocthis](https://bioconductor.org/packages/3.15/biocthis)*.

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    crop = NULL ## Related to https://stat.ethz.ch/pipermail/bioc-devel/2020-April/016656.html
)

## ----"install", eval = FALSE--------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE)) {
#       install.packages("BiocManager")
#   }
# 
# BiocManager::install("planttfhunter")

## ----load_package, message = FALSE--------------------------------------------
library(planttfhunter)

## ----data---------------------------------------------------------------------
data(gsu)
gsu

## ----scheme, echo = FALSE-----------------------------------------------------
data(classification_scheme)
knitr::kable(classification_scheme)

## ----identifying_tfs----------------------------------------------------------
data(gsu_annotation)

# Annotate TF-related domains using a local installation of HMMER
if(hmmer_is_installed()) {
  gsu_annotation <- annotate_pfam(gsu)
} 

# Take a look at the first few lines of the output
head(gsu_annotation)

## ----classifying_tfs----------------------------------------------------------
# Classify TFs into families
gsu_families <- classify_tfs(gsu_annotation)

# Take a look at the output
head(gsu_families)

# Count number of TFs per family
table(gsu_families$Family)

## ----simulate_data_proteomes--------------------------------------------------
set.seed(123) # for reproducibility

# Simulate 4 different species by sampling 100 random genes from `gsu`
proteomes <- list(
    Gsu1 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu2 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu3 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu4 = gsu[sample(names(gsu), 50, replace = FALSE)]
)
proteomes

## ----simulate_data_species_metadata-------------------------------------------
# Create simulated species metadata
species_metadata <- data.frame(
    row.names = names(proteomes),
    Division = "Rhodophyta",
    Origin = c("US", "Belgium", "China", "Brazil")
)

species_metadata

## ----get_tf_counts------------------------------------------------------------
data(tf_counts)

# Get TF counts per family in each species as a SummarizedExperiment object
if(hmmer_is_installed()) {
    tf_counts <- get_tf_counts(proteomes, species_metadata)
}

# Take a look at the SummarizedExperiment object
tf_counts

# Look at the matrix of counts: assay() function from SummarizedExperiment
SummarizedExperiment::assay(tf_counts)

# Look at the species metadata: colData() function from SummarizedExperiment
SummarizedExperiment::colData(tf_counts)

## ----session_info-------------------------------------------------------------
sessioninfo::session_info()

#' Read and parse hmmsearch output
#'
#' @param path Path to hmmsearch output in --domtblout format.
#'
#' @return A parsed data frame with the 23 columns HMMER outputs by default.
#' 
#' @noRd
#' @importFrom utils read.csv
read_hmmsearch <- function(path = NULL) {
  
  stab <- gsub(pattern = "  *", replacement = "\t", readLines(path))
  stab <- stab[!startsWith(stab, "#")]
  if(length(stab) != 0) {
    stab <- paste0(stab, collapse = "\n")
    stab <- utils::read.csv(textConnection(stab), sep = "\t", header = FALSE)
    colnames(stab) <- c(
      "domain_name", "domain_accession", "domain_len",
      "query_name", "query_accession", "qlen",
      "sequence_evalue", "sequence_score", "sequence_bias",
      "domain_N", "domain_of", "domain_cevalue", "domain_ievalue",
      "domain_score", "domain_bias",
      "hmm_from", "hmm_to",
      "ali_from", "ali_to",
      "env_from", "env_to",
      "acc", "description"
    )
  } else {
    stab <- NULL
  }
  return(stab)
}

#' Filter HMMER results based on domain cutoffs for each domain
#'
#' @param hmmer_results A data frame with HMMER results as returned 
#' by \code{read_hmmsearch}.
#' @param evalue Numeric indicating the E-value threshold for hmmsearch 
#' to be used for domains without pre-defined domain cutoffs. 
#' Only valid if parameter mode = 'local'. Default: 1e-05.
#' 
#' @return The same data frame passed as input, but filtered based
#' on domain cutoffs.
#' @noRd
filter_hmmer <- function(hmmer_results = NULL, evalue = 1e-05) {
    
    cutoffs <- score_cutoff
    
    res_cutoff <- merge(hmmer_results, cutoffs,
                        by.x = "query_name", by.y = "domain")
    
    # 1) Filter by domain cutoff
    res_filt1 <- res_cutoff[!is.na(res_cutoff$domaincutoff), ]
    res_filt1 <- res_filt1[res_filt1$domain_score >= res_filt1$domaincutoff, ]
    
    # 2) Filter by evalue
    res_filt2 <- res_cutoff[is.na(res_cutoff$domaincutoff), ]
    res_filt2 <- res_filt2[res_filt2$sequence_evalue < evalue, ]
    
    # Combine results
    res_filtered <- rbind(res_filt1, res_filt2) 
    return(res_filtered)
}


#' Wrapper to check if command is found in PATH
#' 
#' @param cmd Command to test.
#' @param args Arguments for command.
#'
#' @return Logical indicating whether the command is in PATH or not.
#' @noRd
is_valid_cmd <- function(cmd = NULL, args = NULL) {
  found <- tryCatch(
    system2(cmd, args = args, stdout = FALSE, stderr = FALSE), 
    error = function(e) return(FALSE),
    warning = function(w) return(FALSE)
  )
  if(!isFALSE(found)) {
    found <- TRUE
  }
  return(found)
}

#' Check if HMMER is installed
#' 
#' @return Logical indicating whether HMMER is installed or not.
#' @export
#' @rdname hmmer_is_installed
#' @examples 
#' hmmer_is_installed()
hmmer_is_installed <- function() {
  valid <- is_valid_cmd(cmd = "hmmsearch", args = "-h")
  return(valid)
}

#---Load and create data for testing--------------------------------------------
## Example output of hmmsearch (.txt format)
example_output <- c(
    "#                                                                            --- full sequence --- -------------- this domain -------------   hmm coord   ali coord   env coord", 
    "# target name        accession   tlen query name           accession   qlen   E-value  score  bias   #  of  c-Evalue  i-Evalue  score  bias  from    to  from    to  from    to  acc description of target", 
    "#------------------- ---------- ----- -------------------- ---------- ----- --------- ------ ----- --- --- --------- --------- ------ ----- ----- ----- ----- ----- ----- ----- ---- ---------------------", 
    "gsu06140.1           -            340 G2-like              -             56   5.5e-09   24.7   0.0   1   1   2.1e-09   1.1e-08   23.8   0.0     1    53    95   141    95   144 0.89 -", 
    "gsu06160.1           -            120 NF-YC                -            102   1.1e-07   20.5   0.0   1   1   2.7e-08   1.3e-07   20.2   0.0    19    90    18    89     7    98 0.90 -"
)
file <- file.path(tempdir(), "example_file_unit_test.txt")
example_file <- writeLines(
    example_output, file
)

#----Start tests----------------------------------------------------------------
test_that("read_hmmsearch() reads and parses hmmsearch output", {
    output <- read_hmmsearch(file)
    
    expect_equal(ncol(output), 23)
    expect_equal(nrow(output), 2)
    expect_equal(class(output), "data.frame")
})

test_that("filter_hmmer() filters hmmsearch output", {
    output <- read_hmmsearch(file)
    foutput <- filter_hmmer(output)
    
    expect_equal(ncol(foutput), 24)
    expect_equal(nrow(foutput), 1)
    expect_equal(class(foutput), "data.frame")
    expect_equal(
        names(foutput), 
        c(
            "query_name", "domain_name", "domain_accession", "domain_len", 
            "query_accession", "qlen", "sequence_evalue", "sequence_score", 
            "sequence_bias", "domain_N", "domain_of", "domain_cevalue", 
            "domain_ievalue", "domain_score", "domain_bias", "hmm_from", 
            "hmm_to", "ali_from", "ali_to", "env_from", "env_to", "acc", 
            "description", "domaincutoff"
        )
    )
})

test_that("is_valid_cmd() works", {
    v <- is_valid_cmd(cmd = "hmmsearch", args = "-h")
    expect_equal(class(v), "logical")
    expect_equal(length(v), 1)
})

test_that("hmmer_is_installed() works", {
    h <- hmmer_is_installed()
    expect_equal(class(h), "logical")
    expect_equal(length(h), 1)
})

#----Load data------------------------------------------------------------------
data(gsu)
seq <- gsu[1:2]

#----Start tests----------------------------------------------------------------
test_that("annotate_pfam() returns a data frame of domain annotation", {
    
    annot_local <- data.frame(
        Gene = c("gsu04730.1", "gsu00340.1"),
        Domain = c("PF02042", "PF00076")
    )
    if(hmmer_is_installed()) {
        annot_local <- annotate_pfam(seq)
    }
    
    expect_equal(class(annot_local), "data.frame")
    expect_equal(ncol(annot_local), 2)
    expect_equal(names(annot_local), c("Gene", "Domain"))
    expect_true(nrow(annot_local) >= 2)
})

#----Load data------------------------------------------------------------------
set.seed(123)
data(gsu)
data(tf_counts)

# Pick random subsets of 100 genes to simulate other species
proteomes <- list(
    Gsu1 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu2 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu3 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu4 = gsu[sample(names(gsu), 50, replace = FALSE)]
)

# Create species metadata
species_metadata <- data.frame(
    row.names = names(proteomes),
    Division = "Rhodophyta",
    Origin = c("US", "Belgium", "China", "Brazil")
)


#----Start tests----------------------------------------------------------------
test_that("get_tf_counts() returns a SummarizedExperiment object", {
    
    se <- tf_counts
    if(hmmer_is_installed()) {
        se <- get_tf_counts(proteomes, species_metadata)
    }

    expect_error(
        get_tf_counts(proteomes = data.frame(), species_metadata)
    )
    
    expect_error(
        get_tf_counts(proteomes, species_metadata[1:2, ])
    )
    
    expect_true("SummarizedExperiment" %in% class(se))
})

#' Identify TFs and classify them in families
#' 
#' @param domain_annotation A 2-column data frame with the gene ID in the first
#' column and the domain ID in the second column.
#' 
#' @return A 2-column data frame with the variables \strong{Gene} 
#' and \strong{Family} representing gene ID and TF family, respectively.
#' @export
#' @rdname classify_tfs
#' @examples 
#' data(gsu_annotation)
#' domain_annotation <- gsu_annotation
#' families <- classify_tfs(domain_annotation)
classify_tfs <- function(domain_annotation = NULL) {
    
    dom <- c(
        list_domains("dbd"), list_domains("auxiliary"), 
        list_domains("forbidden")
    )
    annot <- domain_annotation[domain_annotation$Domain %in% dom, ]
    annot_list <- split(annot, annot[, 1])
    
    fams <- Reduce(rbind, lapply(annot_list, function(x) {
        domains <- x$Domain
        
        # Apply all helper classification functions to object domains
        funcs <- list(
            check_ap2_erf, check_b3, check_c2c2, check_garp,
            check_hb, check_mads, check_myb, check_nf_y, check_smallfams
        )
        tffam <- unlist(lapply(funcs, function(f) f(domains)))
        if(length(tffam) == 0) {
            fam_df <- NULL
        } else {
            fam_df <- data.frame(
                Gene = unique(x[, 1]),
                Family = tffam
            )
            
        }
        return(fam_df)
    }))
    return(fams)
}

#' Annotate proteins sequences with PFAM domains
#' 
#' PFAM domains are assigned to each sequence using HMMER.
#' 
#' @param seq An AAStringSet object as returned 
#' by \code{Biostrings::readAAStringSet()}. The sequences in this object must 
#' represent only the translated sequences of primary (or longest) transcripts.
#' @param evalue Numeric indicating the E-value threshold for hmmsearch 
#' to be used for domains without pre-defined domain cutoffs. 
#' Only valid if parameter mode = 'local'. Default: 1e-05.
#'
#' @return A 2-column data frame with the variables \strong{Gene} 
#' and \strong{Domain}, which contain gene IDs and domain IDs, respectively.
#' 
#' @importFrom Biostrings writeXStringSet
#' @export
#' @rdname annotate_pfam
#' @examples 
#' data(gsu)
#' seq <- gsu[1:5]
#' if(hmmer_is_installed()) {
#'     annotate_pfam(seq)
#' }
annotate_pfam <- function(seq = NULL, evalue = 1e-05) {
    
    if(!hmmer_is_installed()) {
        stop("Could not find HMMER. Check if it is installed and in your PATH.")
    }
    
    # Write file to temporary directory
    seq_path <- tempfile("seq", fileext = ".fasta")
    Biostrings::writeXStringSet(seq, filepath = seq_path)
    
    # Run hmmsearch using pre-built HMM profiles in extdata/
    profiles <- system.file("extdata", package = "planttfhunter")
    profiles <- list.files(profiles, pattern = ".hmm", full.names = TRUE)
    
    annotation <- lapply(profiles, function(x) {
        out_file <- tempfile(pattern = "search", fileext = ".txt")
        cmd_args <- c("--domtblout", out_file, x, seq_path)
        system2("hmmsearch", args = cmd_args, stdout = FALSE)
        result <- read_hmmsearch(path = out_file)
        annot <- NULL
        if(!is.null(result)) {
            result <- filter_hmmer(result)
            
            if(nrow(result) != 0) {
                annot <- data.frame(
                    Gene = result$domain_name, 
                    Domain = result$query_name
                )
            }
        }
        unlink(out_file)
        return(annot)
    })
    annotation <- Reduce(rbind, annotation)
    unlink(seq_path)
    return(annotation)
}

#' Get TF frequencies for each species as a SummarizedExperiment object
#'
#' This function identifies and classifies TFs, and returns TF counts for 
#' each family as a SummarizedExperiment object
#'
#' @param proteomes List of \strong{AAStringSet} objects
#' @param species_metadata (Optional) A data frame containing species names 
#' in row names (names must match element names in 
#' the \strong{proteomes} list), and species metadata 
#' (e.g., taxonomic information, ecological information)
#' in columns. If NULL, the colData of the SummarizedExperiment object
#' will be empty.
#'
#' @return A SummarizedExperiment object containing transcription factor
#' frequencies per family in each species, as well as species metadata (if
#' \strong{species_metadata} is not NULL).
#' 
#' @export
#' @rdname get_tf_counts
#' @importFrom SummarizedExperiment SummarizedExperiment
#' @importFrom methods is
#' @examples 
#' data(gsu)
#' 
#' set.seed(123)
#' # Pick random subsets of 100 genes to simulate other species
#' proteomes <- list(
#'     Gsu1 = gsu[sample(names(gsu), 50, replace = FALSE)],
#'     Gsu2 = gsu[sample(names(gsu), 50, replace = FALSE)],
#'     Gsu3 = gsu[sample(names(gsu), 50, replace = FALSE)],
#'     Gsu4 = gsu[sample(names(gsu), 50, replace = FALSE)]
#' )
#' 
#' # Create species metadata
#' species_metadata <- data.frame(
#'     row.names = names(proteomes),
#'     Division = "Rhodophyta",
#'     Origin = c("US", "Belgium", "China", "Brazil")
#' )
#' 
#' # Get SummarizedExperiment object
#' if(hmmer_is_installed()) {
#'     se <- get_tf_counts(proteomes, species_metadata)
#' }
get_tf_counts <- function(proteomes, species_metadata = NULL) {
    
    # Check 1: is `proteomes` a list of AAStringSet objects?
    if(!is(proteomes, "list") || !is(proteomes[[1]], "AAStringSet")) {
        stop("Argument to 'proteomes' must be a list of AAStringSet objects.")
    }
    # Check 2: handle colData depending on `species_metadata`'s class
    coldata <- data.frame(row.names = names(proteomes))
    if(is(species_metadata, "data.frame")) {
        if(!identical(sort(rownames(species_metadata)), names(proteomes))) {
            stop("Names of 'proteomes' must match rownames of 'species_metadata'.")
        }
        coldata <- species_metadata
    }
    
    # Get a list of TFs frequencies per family
    freqs <- lapply(seq_along(proteomes), function(x) {
        
        species <- names(proteomes)[x]
        domains <- annotate_pfam(proteomes[[x]])
        families <- classify_tfs(domains)
        counts <- as.data.frame(table(families$Family))
        names(counts) <- c("Family", species)
        return(counts)
    })
    # Combine list into a quantitative matrix
    freqs_mat <- Reduce(function(x, y) merge(x, y, all = TRUE), freqs)
    rownames(freqs_mat) <- freqs_mat$Family
    freqs_mat$Family <- NULL
    freqs_mat <- as.matrix(freqs_mat)
    freqs_mat[is.na(freqs_mat)] <- 0

    # Create SummarizedExperiment object
    se <- SummarizedExperiment::SummarizedExperiment(
        assays = list(counts = freqs_mat), colData = coldata
    )
    return(se)
}

#' Protein sequences of the algae species Galdieria sulphuraria
#'
#' Data obtained from PLAZA Diatoms. Only genes containing domains used for
#' TF family classification were kept for package size issues.
#'
#' @name gsu
#' @format An AAStringSet object as returned 
#' by \code{Biostrings::readAAStringSet()}.
#' @references
#' Osuna-Cruz, C. M., Bilcke, G., Vancaester, E., De Decker, S., Bones, A. M., 
#' Winge, P., ... & Vandepoele, K. (2020). The Seminavis robusta genome 
#' provides insights into the evolutionary adaptations of benthic diatoms. 
#' Nature communications, 11(1), 1-13.
#' @examples
#' data(gsu)
#' @usage data(gsu)
"gsu"


#' Data frame of TF family classification scheme
#'
#' The classification scheme is the same as the one used by PlantTFDB.
#' 
#' @name classification_scheme
#' @format A data frame with the following variables:
#' \describe{
#'  \item{Family}{TF family name.}
#'  \item{Subfamily}{TF subfamily name.}
#'  \item{DBD}{DNA-binding domain}
#'  \item{Auxiliary}{Auxiliary domain}
#'  \item{Forbidden}{Forbidden domain}
#' }
#' @references 
#' Jin, J., Tian, F., Yang, D. C., Meng, Y. Q., Kong, L., Luo, J., & 
#' Gao, G. (2016). PlantTFDB 4.0: toward a central hub for transcription 
#' factors and regulatory interactions in plants. 
#' Nucleic acids research, gkw982.
#' @examples 
#' data(classification_scheme)
#' @usage data(classification_scheme)
"classification_scheme"


#' Domain annotation for the algae species Galdieria sulphuraria
#'  
#' The data set was created using the funcion \code{annotate_pfam()} in local
#' mode.
#'
#' @name gsu_annotation
#' @format A 2-column data frame with the following variables:
#' \describe{
#'  \item{Gene}{Gene ID}
#'  \item{Annotation}{Domain ID or domain name when ID is not available in PFAM}
#'}
#' @examples
#' data(gsu_annotation)
#' @usage data(gsu_annotation)
"gsu_annotation"


#' TFs families of the algae species Galdieria sulphuraria
#'  
#' The data set was created using the funcion \code{classify_tfs()}.
#'
#' @name gsu_families
#' @format A 2-column data frame with the following variables:
#' \describe{
#'  \item{Gene}{Gene ID}
#'  \item{Family}{TF family}
#'}
#' @examples
#' data(gsu_families)
#' @usage data(gsu_families)
"gsu_families"


#' TF counts per family in 4 simulated species
#' 
#' Simulated species were created by sampling 100 genes from the example
#' data set \strong{gsu} with after \code{set.seed(123)}.
#' 
#' @name tf_counts 
#' @format A SummarizedExperiment with TF frequencies per family in each 
#' species in \strong{assay} and species metadata in \strong{colData}. 
#' @examples
#' data(tf_counts)
#' @usage data(tf_counts)
"tf_counts"

#' List possible domains used for TF classification
#'
#' @param show Character specifying what to show. One of 'dbd', 'auxiliary',
#' 'forbidden', or 'all'. Default: 'dbd'.
#'
#' @return A named character vector of domains and their IDs.
#' @noRd
list_domains <- function(show = "dbd") {
  domains <- c(
    AP2 = "PF00847", B3 = "PF02362", auxin_resp = "PF06507", 
    GAGA_bind = "PF06217", DUF822 = "PF05687", HLH = "PF00010",
    bZIP_1 = "PF00170", zf_B_box = "PF00643", CCT = "PF06203", 
    Zf_Dof = "PF02701", GATA_zf = "PF00320", Zf_LSD1 = "PF06943",
    Peptidase_C14 = "PF00656", YABBY = "PF04690", Zf_C2H2 =  "PF00096",
    RNase_T = "PF00929", Zf_CCCH =  "PF00642", RRM_1 = "PF00076",
    Helicase_C = "PF00271", CG1 = "PF03859", TCR = "PF03638",
    E2F_TDP = "PF02319", EIN3 = "PF04873", FAR1 = "PF03101",
    STAT = "PF02864", Response_reg = "PF00072", DUF573 = "PF04504",
    GRAS = "PF03514", WRC = "PF08879", QLQ = "PF08880",
    Homeobox = "PF00046", SMART = "PF01852", ELK = "PF03789",
    PHD = "PF00628", HSF = "PF00447", DUF260 = "PF03195",
    FLO_LFY = "PF01698", SRF_TF = "PF00319", K_box = "PF01486",
    MYB_DNAbind = "PF00249", SWIRM = "PF04433", NAM = "PF02365",
    Zf_NF_X1 = "PF01422", CBFB_NFYA = "PF02045", RWP_RK = "PF02042",
    NOZZLE = "PF08744", S1FA = "PF04689", SBP = "PF03110",
    DUF702 = "PF05142", TCP = "PF03634", WRKY = "PF03106",
    Whirly = "PF08536", ZF_HD_dimer = "PF04770",
    G2_like = "G2-like", HD_ZIP = "HD-ZIP",
    HRT_like = "HRT-like", NF_YB = "NF-YB", NF_YC = "NF-YC",
    SAP = "SAP", Trihelix = "Trihelix", VOZ = "VOZ", WOX = "WOX"
  )
  aux_domains <- c("PF06507", "PF06203", "PF00072", "PF08880", "PF01852", 
                   "HD-ZIP", "PF03789", "WOX", "PF00628", "PF01486")
  forb_domains <- c("PF04433", "PF00271", "PF00076", "PF00929", "PF00656")
  
  if(show == "forbidden") {
    fdomains <- domains[domains %in% forb_domains]
  } else if(show == "auxiliary") {
    fdomains <- domains[domains %in% aux_domains]
  } else if(show == "dbd") {
    fdomains <- domains[!(domains %in% c(forb_domains, aux_domains))]
  } else {
    stop("Invalid argument to parameter show.")
  }
  return(fdomains)
}


#' Check if TF belongs to the AP2/ERF family
#'
#' @param domains Character scalar or vector of domains for a given gene.
#' 
#' @return A character scalar with the TF family or NULL if it belongs to a
#' different family.
#' @noRd
check_ap2_erf <- function(domains = NULL) {
  len_ap2 <- length(domains[domains == "PF00847"])
  len_b3 <- length(domains[domains == "PF02362"])
  
  if(len_ap2 >= 2) {
    fam <- "AP2"
  } else if(len_ap2 == 1 & len_b3 == 0) {
    fam <- "ERF"
  } else if(len_ap2 != 0 & len_b3 != 0) {
    fam <- "RAV"
  } else {
    fam <- NULL
  }
  return(fam)
}


#' Check if TF belongs to the B3 family
#'
#' @param domains Character scalar or vector of domains for a given gene.
#' 
#' @return A character scalar with the TF family or NULL if it belongs to a
#' different family.
#' @noRd
check_b3 <- function(domains = NULL) {
  len_ap2 <- length(domains[domains == "PF00847"])
  len_auxin <- length(domains[domains == "PF06507"])
  len_b3 <- length(domains[domains == "PF02362"])
  
  if(len_b3 != 0 & len_auxin != 0) {
    fam <- "ARF"
  } else if(len_b3 != 0 & len_ap2 == 0 & len_auxin == 0) {
    fam <- "B3"
  } else {
    fam <- NULL
  }
  return(fam)
}


#' Check if TF belongs to the C2C2 family
#'
#' @param domains Character scalar or vector of domains for a given gene.
#' 
#' @return A character scalar with the TF family or NULL if it belongs to a
#' different family.
#' @noRd
check_c2c2 <- function(domains = NULL) {
  len_zf_bbox <- length(domains[domains == "PF00643"])
  len_cct <- length(domains[domains == "PF06203"])
  len_zf_dof <- length(domains[domains == "PF02701"])
  len_gata_zf <- length(domains[domains == "PF00320"])
  len_zf_lsd1 <- length(domains[domains == "PF06943"])
  len_pep_c14 <- length(domains[domains == "PF00656"])
  len_yabby <- length(domains[domains == "PF04690"])
  
  if(len_zf_bbox != 0 & len_cct != 0) {
    fam <- "CO-like"
  } else if(len_zf_dof != 0) {
    fam <- "Dof"
  } else if(len_gata_zf != 0) {
    fam <- "GATA"
  } else if(len_zf_lsd1 != 0 & len_pep_c14 == 0) {
    fam <- "LSD"
  } else if(len_yabby != 0) {
    fam <- "YABBY"
  } else {
    fam <- NULL
  }
  
  return(fam)
}


#' Check if TF belongs to the GARP family
#'
#' @param domains Character scalar or vector of domains for a given gene.
#' 
#' @return A character scalar with the TF family or NULL if it belongs to a
#' different family.
#' @noRd
check_garp <- function(domains = NULL) {
    len_g2like <- length(domains[domains == "G2-like"])
    len_resp_rep <- length(domains[domains == "PF00072"])
    
    if(len_g2like != 0 & len_resp_rep != 0) {
        fam <- "ARR-B"
    } else if(len_g2like != 0 & len_resp_rep == 0) {
        fam <- "G2-like"
    } else {
        fam <- NULL
    }
    
    return(fam)
}


#' Check if TF belongs to the HB family
#'
#' @param domains Character scalar or vector of domains for a given gene.
#' 
#' @return A character scalar with the TF family or NULL if it belongs to a
#' different family.
#' @noRd
check_hb <- function(domains = NULL) {
    len_homeobox <- length(domains[domains == "PF00046"])
    len_hdzip <- length(domains[domains == "HD-ZIP_I/II"])
    len_smart <- length(domains[domains == "PF01852"])
    len_elk <- length(domains[domains == "PF03789"])
    len_bell <- length(domains[domains == "BELL"])
    len_wox <- length(domains[domains == "Wus_type_Homeobox"])
    len_phd <- length(domains[domains == "PF00628"])
    only_homeobox <- unique(domains)
    only_homeobox <- length(only_homeobox) == 1 & "PF00046" %in% only_homeobox
    
    if(len_homeobox != 0 & len_hdzip != 0) {
        fam <- "HD-ZIP"
    } else if(len_homeobox != 0 & len_smart != 0) {
        fam <- "HD-ZIP"
    } else if(len_homeobox != 0 & len_elk != 0) {
        fam <- "TALE"
    } else if(len_homeobox != 0 & len_bell != 0) {
        fam <- "TALE"
    } else if(len_homeobox != 0 & len_wox != 0) {
        fam <- "WOX"
    } else if(len_homeobox != 0 & len_phd != 0) {
        fam <- "HB-PHD"
    } else if(only_homeobox) {
        fam <- "HB-other"
    } else {
        fam <- NULL
    }
    
    return(fam)
}


#' Check if TF belongs to the MADS-box family
#'
#' @param domains Character scalar or vector of domains for a given gene.
#' 
#' @return A character scalar with the TF family or NULL if it belongs to a
#' different family.
#' @noRd
check_mads <- function(domains = NULL) {
    len_srf_tf <- length(domains[domains == "PF00319"])
    len_kbox <- length(domains[domains == "PF01486"])
    
    if(len_srf_tf != 0 & len_kbox == 0) {
        fam <- "M-type"
    } else if(len_srf_tf != 0 & len_kbox != 0) {
        fam <- "MIKC"
    } else {
        fam <- NULL
    }
    
    return(fam)
}


#' Check if TF belongs to the MYB family
#'
#' @param domains Character scalar or vector of domains for a given gene.
#' 
#' @return A character scalar with the TF family or NULL if it belongs to a
#' different family.
#' @noRd
check_myb <- function(domains = NULL) {
    len_myb <- length(domains[domains == "PF00249"])
    len_swirm <- length(domains[domains == "PF04433"])
    
    if(len_myb == 1 & len_swirm == 0) {
        fam <- "MYB-related"
    } else if(len_myb > 1 & len_swirm == 0) {
        fam <- "MYB"
    } else {
        fam <- NULL
    }
    
    return(fam)
}


#' Check if TF belongs to the NF-Y family
#'
#' @param domains Character scalar or vector of domains for a given gene.
#' 
#' @return A character scalar with the TF family or NULL if it belongs to a
#' different family.
#' @noRd
check_nf_y <- function(domains = NULL) {
    len_cbfb_nfya <- length(domains[domains == "PF02045"])
    len_nf_yb <- length(domains[domains == "NF-YB"])
    len_nf_yc <- length(domains[domains == "NF-YC"])
    
    if(len_cbfb_nfya != 0) {
        fam <- "NF-YA"
    } else if(len_nf_yb != 0) {
        fam <- "NF-YB"
    } else if(len_nf_yc != 0) {
        fam <- "NF-YC"
    } else {
        fam <- NULL
    }
    
    return(fam)
}


#' Check if TF belongs to one of the smaller families
#'
#' @param domains Character scalar or vector of domains for a given gene.
#' 
#' @return A character scalar with the TF family or NULL if it belongs to a
#' different family.
#' @noRd
check_smallfams <- function(domains = NULL) {
    
    count <- function(x) { return(length(domains[domains == x])) }
    
    no_c3h_forbidden <- count("PF00076") == 0 & count("PF00271") == 0
    
    if(count("PF06217") != 0) { # GAGA_bind
        fam <- "BBR-BPC"
    } else if(count("PF05687") != 0) { # DUF822
        fam <- "BES1"
    } else if(count("PF00010") != 0) { # HLH
        fam <- "bHLH"
    } else if(count("PF00170") != 0) { # bZIP_1
        fam <- "bZIP"
    } else if(count("PF00096") != 0 & count("PF00929") == 0) { # zf-C2H2, RNase_T
        fam <- "C2H2"
    } else if(count("PF00642") != 0 & no_c3h_forbidden) { 
        fam <- "C3H"
    } else if(count("PF03859") != 0) { # CG1
        fam <- "CAMTA"
    } else if(count("PF03638") != 0) { # TCR
        fam <- "CPP"
    } else if(count("PF00643") > 1) { # zf-B_box
        fam <- "DBB"
    } else if(count("PF02319") != 0) { # E2F_TDP
        fam <- "E2F/DP"
    } else if(count("PF04873") != 0) { # EIN3
        fam <- "EIL"
    } else if(count("PF03101") != 0) { # FAR1
        fam <- "FAR1"
    } else if(count("PF04504") != 0) { # DUF573
        fam <- "GeBP"
    } else if(count("PF03514") != 0) { # GRAS
        fam <- "GRAS"
    } else if(count("PF08879") != 0 & count("PF08880") != 0) { # WRC, QLQ
        fam <- "GRF"
    } else if(count("HRT-like") != 0) { # HRT-like, self-built
        fam <- "HRT-like"
    } else if(count("PF00447") != 0) { # HSF_dna_bind
        fam <- "HSF"
    } else if(count("PF03195") != 0) { # DUF260
        fam <- "LBD"
    } else if(count("PF01698") != 0) { # FLO_LFY
        fam <- "LFY"
    } else if(count("PF02365") != 0) { # NAM
        fam <- "NAC"
    } else if(count("PF01422") != 0) { # zf-NF-X1
        fam <- "NF-X1"
    } else if(count("PF02042") != 0) { # RWP-RK
        fam <- "Nin-like"
    } else if(count("PF08744") != 0) { # NOZZLE
        fam <- "NZZ/SPL"
    } else if(count("PF04689") != 0) { # S1FA
        fam <- "S1Fa-like"
    } else if(count("SAP") != 0) { # SAP
        fam <- "SAP"
    } else if(count("PF03110") != 0) { # SBP
        fam <- "SBP"
    } else if(count("PF05142") != 0) { # DUF702
        fam <- "SRS"
    } else if(count("STAT") != 0) { # STAT
        fam <- "STAT"
    } else if(count("PF03634") != 0) { # TCP
        fam <- "TCP"
    } else if(count("trihelix") != 0) { # Trihelix
        fam <- "Trihelix"
    } else if(count("VOZ") != 0) { # VOZ
        fam <- "VOZ"
    } else if(count("PF08536") != 0) { # Whirly
        fam <- "Whirly"
    } else if(count("PF03106") != 0) { # WRKY
        fam <- "WRKY"
    } else if(count("PF04770") != 0) { # ZF-HD_dimer
        fam <- "ZF-HD"
    } else {
        fam <- NULL
    }
    return(fam)
}

#----Load data------------------------------------------------------------------
data(gsu_annotation)
domain_annotation <- gsu_annotation

#----Start tests----------------------------------------------------------------
test_that("classify_tfs() returns a 2-column data frame", {
  families <- classify_tfs(domain_annotation)
  expect_equal(class(families), "data.frame")
  expect_equal(ncol(families), 2)
})

test_that("list_domains() returns a named character vector", {
    dom_dbd <- list_domains(show = "dbd")
    dom_forbidden <- list_domains(show = "forbidden")
    dom_aux <- list_domains(show = "auxiliary")
    
    expect_equal(length(dom_dbd), 47)
    expect_equal(length(dom_forbidden), 5)
    expect_equal(length(dom_aux), 10)
    expect_equal(class(dom_dbd), "character")
    expect_equal(class(dom_forbidden), "character")
    expect_equal(class(dom_aux), "character")
    expect_error(list_domains(show = "anything"))
})


test_that("check_ap2_erf() returns domain classification", {
    ct <- check_ap2_erf("PF00847")
    ct2 <- check_ap2_erf(rep("PF00847", 2))
    ct3 <- check_ap2_erf(c("PF02362", "PF00847"))
    cf <- check_ap2_erf("fakedomain")
    
    expect_equal(ct, "ERF")
    expect_equal(ct2, "AP2")
    expect_equal(ct3, "RAV")
    expect_equal(cf, NULL)
})


test_that("check_b3() returns domain classification", {
    ct <- check_b3("PF02362")
    ct2 <- check_b3(c("PF06507", "PF02362"))
    cf <- check_b3("fakedomain")
    
    expect_equal(ct, "B3")
    expect_equal(ct2, "ARF")
    expect_equal(cf, NULL)
})


test_that("check_c2c2() returns domain classification", {
    ct <- check_c2c2("PF00320")
    ct2 <- check_c2c2(c("PF00643", "PF06203"))
    ct3 <- check_c2c2("PF02701")
    ct4 <- check_c2c2("PF06943")
    ct5 <- check_c2c2("PF04690")
    cf <- check_c2c2("fakedomain")
    
    expect_equal(ct, "GATA")
    expect_equal(ct2, "CO-like")
    expect_equal(ct3, "Dof")
    expect_equal(ct4, "LSD")
    expect_equal(ct5, "YABBY")
    expect_equal(cf, NULL)
})


test_that("check_garp() returns domain classification", {
    ct <- check_garp("G2-like")
    ct2 <- check_garp(c("PF00072", "G2-like"))
    cf <- check_garp("fakedomain")
    
    expect_equal(ct, "G2-like")
    expect_equal(ct2, "ARR-B")
    expect_equal(cf, NULL)
})


test_that("check_hb() returns domain classification", {
    ct <- check_hb("PF00046")
    ct2 <- check_hb(c("PF00046", "HD-ZIP_I/II"))
    ct3 <- check_hb(c("PF03789", "PF00046"))
    ct4 <- check_hb(c("PF00628", "PF00046"))
    ct5 <- check_hb(c("BELL", "PF00046"))
    ct6 <- check_hb(c("Wus_type_Homeobox", "PF00046"))
    ct7 <- check_hb(c("PF01852", "PF00046"))
    cf <- check_hb("fakedomain")
    
    expect_equal(ct, "HB-other")
    expect_equal(ct2, "HD-ZIP")
    expect_equal(ct3, "TALE")
    expect_equal(ct4, "HB-PHD")
    expect_equal(ct5, "TALE")
    expect_equal(ct6, "WOX")
    expect_equal(ct7, "HD-ZIP")
    expect_equal(cf, NULL)
})


test_that("check_mads() returns domain classification", {
    ct <- check_mads("PF00319")
    ct2 <- check_mads(c("PF01486", "PF00319"))
    cf <- check_mads("fakedomain")
    
    expect_equal(ct, "M-type")
    expect_equal(ct2, "MIKC")
    expect_equal(cf, NULL)
})


test_that("check_myb() returns domain classification", {
    ct <- check_myb("PF00249")
    cf <- check_myb("fakedomain")
    
    expect_equal(ct, "MYB-related")
    expect_equal(cf, NULL)
})


test_that("check_nf_y() returns domain classification", {
    ct <- check_nf_y("PF02045")
    cf <- check_nf_y("fakedomain")
    
    expect_equal(ct, "NF-YA")
    expect_equal(cf, NULL)
})


test_that("check_smallfams() returns domain classification", {
    ct <- check_smallfams("PF00010")
    ct2 <- check_smallfams("PF06217")
    ct3 <- check_smallfams("PF05687")
    ct4 <- check_smallfams("PF00642")
    ct5 <- check_smallfams("PF03859")
    ct6 <- check_smallfams(rep("PF00643", 2))
    ct7 <- check_smallfams("PF04873")
    ct8 <- check_smallfams("PF03101")
    ct9 <- check_smallfams("PF04504")
    ct10 <- check_smallfams("PF03514")
    ct11 <- check_smallfams(c("PF08879", "PF08880"))
    ct12 <- check_smallfams("HRT-like")
    ct13 <- check_smallfams("PF03195")
    ct14 <- check_smallfams("PF01698")
    ct15 <- check_smallfams("PF02365")
    ct16 <- check_smallfams("PF08744")
    ct17 <- check_smallfams("PF04689")
    ct18 <- check_smallfams("SAP")
    ct19 <- check_smallfams("PF03110")
    ct20 <- check_smallfams("PF05142")
    ct21 <- check_smallfams("STAT")
    ct22 <- check_smallfams("PF03634")
    ct23 <- check_smallfams("trihelix")
    ct24 <- check_smallfams("VOZ")
    ct25 <- check_smallfams("PF08536")
    ct26 <- check_smallfams("PF03106")
    ct27 <- check_smallfams("PF04770")
    cf <- check_smallfams("fakedomain")
    
    expect_equal(ct, "bHLH")
    expect_equal(ct2, "BBR-BPC")
    expect_equal(ct3, "BES1")
    expect_equal(ct4, "C3H")
    expect_equal(ct5, "CAMTA")
    expect_equal(ct6, "DBB")
    expect_equal(ct7, "EIL")
    expect_equal(ct8, "FAR1")
    expect_equal(ct9, "GeBP")
    expect_equal(ct10, "GRAS")
    expect_equal(ct11, "GRF")
    expect_equal(ct12, "HRT-like")
    expect_equal(ct13, "LBD")
    expect_equal(ct14, "LFY")
    expect_equal(ct15, "NAC")
    expect_equal(ct16, "NZZ/SPL")
    expect_equal(ct17, "S1Fa-like")
    expect_equal(ct18, "SAP")
    expect_equal(ct19, "SBP")
    expect_equal(ct20, "SRS")
    expect_equal(ct21, "STAT")
    expect_equal(ct22, "TCP")
    expect_equal(ct23, "Trihelix")
    expect_equal(ct24, "VOZ")
    expect_equal(ct25, "Whirly")
    expect_equal(ct26, "WRKY")
    expect_equal(ct27, "ZF-HD")

    expect_equal(cf, NULL)
})

Data creation
================

# Data in data/

## gsu.rda

This object contains the protein sequences of the algae *Galdieria
sulphuraria*, and it was downloaded from PLAZA Diatoms. Only genes that
have domains of interest (used for classification) were kept for package
size issues.

``` r
gsu <- Biostrings::readAAStringSet(
  "ftp://ftp.psb.ugent.be/pub/plaza/plaza_diatoms_01/Fasta/proteome.selected_transcript.gsu.fasta.gz"
)
names(gsu) <- gsub("\ .*", "", names(gsu))

# The code below must be run after identifying TF domains with annotate_pfam()
data("gsu_annotation")
gsu <- gsu[names(gsu) %in% gsu_annotation$Gene]

outfile <- paste0(tempdir(), "/gsu.fasta")
Biostrings::writeXStringSet(gsu, filepath = outfile)
gsu <- Biostrings::readAAStringSet(outfile)

usethis::use_data(
  gsu, compress = "xz", overwrite = TRUE
)
```

## gsu_annotation.rda

``` r
data(gsu)
gsu_annotation <- annotate_pfam(gsu, mode = "local", evalue = 1e-05)

usethis::use_data(
  gsu_annotation, compress = "xz", overwrite = TRUE
)
```

## classification_scheme.rda

The classification scheme is the same as implemented in PlantTFDB.

``` r
library(tidyverse)
table <- rvest::read_html("http://planttfdb.gao-lab.org/help_famschema.php") %>%
  rvest::html_table()
table <- as.data.frame(table[[3]])

table <- table[-1, -1]
names(table) <- c("Family", "Subfamily", "DBD", "Auxiliary", "Forbidden")
table[table == ""] <- NA

table$DBD <- gsub("\\r\\n\\t\\t\\r\\n\\t\\t", " or ", table$DBD)
table$DBD <- gsub("\ \\(self-build\\)", "", table$DBD)
table$Auxiliary <- gsub("\ \\(self-build\\)", "", table$Auxiliary)
table$Auxiliary <- gsub("BELLor", "BELL or", table$Auxiliary)
table$Forbidden <- gsub("\\r\\n\\t\\tor", " or ", table$Forbidden)

classification_scheme <- table
classification_scheme$DBD[3] <- gsub("or", "and", classification_scheme$DBD[3])

usethis::use_data(classification_scheme,
                  compress = "xz", overwrite = TRUE)
```

## gsu_families.rda

``` r
gsu_families <- classify_tfs(gsu_annotation)

usethis::use_data(gsu_families, compress = "xz", overwrite = TRUE)
```

## tf_counts.rda

This data set contains a `SummarizedExperiment` of motif counts for each
family in simulated species.

``` r
data(gsu)
set.seed(123) # for reproducibility

# Simulate 5 different species by sampling 100 random genes from `gsu`
proteomes <- list(
    Gsu1 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu2 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu3 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu4 = gsu[sample(names(gsu), 50, replace = FALSE)]
)

# Create simulated species metadata
species_metadata <- data.frame(
    row.names = names(proteomes),
    Division = "Rhodophyta",
    Origin = c("US", "Belgium", "China", "Brazil")
)

tf_counts <- get_tf_counts(proteomes, species_metadata)

usethis::use_data(tf_counts, compress = "xz")
```

# Data in inst/extdata

## \*.hmm files (profile HMMs)

The seed alignments for all TF families were downloaded from PFAM and
converted to profile HMMs using the command `hmmbuild` from HMMER.
Domains that were self-built by PlantTFDB were kindly provided by the
database maintainers by email.

``` r
library(tidyverse)
domains <- c(
  AP2 = "PF00847", B3 = "PF02362", auxin_resp = "PF06507", 
  GAGA_bind = "PF06217", DUF822 = "PF05687", HLH = "PF00010",
  bZIP_1 = "PF00170", zf_B_box = "PF00643", CCT = "PF06203", 
  Zf_Dof = "PF02701", GATA_zf = "PF00320", Zf_LSD1 = "PF06943",
  Peptidase_C14 = "PF00656", YABBY = "PF04690", Zf_C2H2 =  "PF00096",
  RNase_T = "PF00929", Zf_CCCH =  "PF00642", RRM_1 = "PF00076",
  Helicase_C = "PF00271", CG1 = "PF03859", TCR = "PF03638",
  E2F_TDP = "PF02319", EIN3 = "PF04873", FAR1 = "PF03101",
  STAT = "PF02864", Response_reg = "PF00072", DUF573 = "PF04504",
  GRAS = "PF03514", WRC = "PF08879", QLQ = "PF08880",
  Homeobox = "PF00046", SMART = "PF01852", ELK = "PF03789",
  PHD = "PF00628", HSF = "PF00447", DUF260 = "PF03195",
  FLO_LFY = "PF01698", SRF_TF = "PF00319", K_box = "PF01486",
  MYB_DNAbind = "PF00249", SWIRM = "PF04433", NAM = "PF02365",
  Zf_NF_X1 = "PF01422", CBFB_NFYA = "PF02045", RWP_RK = "PF02042",
  NOZZLE = "PF08744", S1FA = "PF04689", SBP = "PF03110",
  DUF702 = "PF05142", TCP = "PF03634", WRKY = "PF03106",
  Whirly = "PF08536", ZF_HD_dimer = "PF04770"
)

# Define function to download seed alignments for PFAM domains
download_pfam_seeds <- function(domains = NULL) {
  
  d <- lapply(domains, function(x) {
    url <- paste0('http://pfam.xfam.org/family/', x,
                  '/alignment/seed/format?format=fasta')
    aln_path <- paste0(tempdir(), "/", x, ".fasta")
    if(!file.exists(aln_path)) {
      options(timeout = 5)
      result <- tryCatch({
        downloader::download(url = url, destfile = aln_path)
        res <- TRUE
      },
      error = function(e) return(FALSE),
      warning = function(w) return(FALSE)
      )
      
      if (!result) {
        file.remove(aln_path)
      }
    }
    return(NULL)
  })
  files <- list.files(tempdir(), pattern = ".fasta")
  ds <- sapply(domains, function(x) return(TRUE %in% 
                                             stringr::str_detect(files, x)))
  status <- data.frame(
    domain = domains,
    download_status = ds
  )
  return(status)
}

download_aln <- download_pfam_seeds(domains)


# Define function to build profile HMMs from alignments
build_profile_HMM <- function(domains = NULL) {
  
  f <- lapply(domains, function(x) {
    
    aln_path <- paste0(tempdir(), "/", x, ".fasta")
    hmm_path <- paste0(tempdir(), "/", x, ".hmm")
    build <- system2("hmmbuild", args = c(hmm_path, aln_path))
    sedargs <- paste0("s/aln$/", x, "/g")
    rename_aln <- system2("sed", args = c("-i", sedargs, hmm_path))
    return(NULL)
  })
  files <- list.files(tempdir(), pattern = ".hmm")
  build <- sapply(domains, function(x) return(TRUE %in% 
                                             stringr::str_detect(files, x)))
  status_df <- data.frame(
    domain = domains,
    build_status = build
  )
  return(status_df)
}

build_profile_HMM(domains)

# Combine .hmm files in a single one
combine_domains <- function() {
  files <- list.files(tempdir(), pattern = ".hmm", full.names = TRUE)
  files <- paste0(files, collapse = " ")
  out_hmm <- file.path(tempdir(), "PFAM.hmm")
  system2("cat", args = c(files, " > ", out_hmm))
  return(NULL)
}

combine_domains()
fs::file_move(path = file.path(tempdir(), "PFAM.hmm"),
              new_path = here::here("inst", "extdata", "PFAM.hmm"))
```

# Internal data (sysdata.rda)

This data frame contains the sequence and domain cutoff for each family.
The values were retrieved from the Supplementary Materials of the
PlantTFDB v2 paper.

``` r
library(here)

# Create a list of PFAM domains
pfam_domains <- readLines(here("inst", "extdata", "PFAM.hmm"))
pfam_domains <- pfam_domains[grep("^NAME", pfam_domains)]
pfam_domains <- gsub(".*PF", "PF", pfam_domains)
    
# Create a list of self-built domains
selfbuilt <- readLines(here("inst", "extdata", "self_built.hmm"))
selfbuilt <- selfbuilt[grep("^NAME", selfbuilt)]
selfbuilt <- gsub("NAME  ", "", selfbuilt)
    

score_cutoff <- data.frame(domain = c(pfam_domains, selfbuilt))

score_cutoff$domaincutoff <- c(
    11, 15, 20, NA, 12.2, 19.7, 21.5, NA, 21, 22, # 1:10
    22.5, NA, 17, 16, NA, NA, NA, 16, NA, 20.7, # 11:20
    NA, 20.8, 22, 23, 30.2, 21.5, 26, 150, 21.3, 22, # 21:30
    23, 22, 20.5, 21, 20.9, NA, 25, NA, 29.5, 22, # 31:40
    21, 20.5, 22, 21.3, 25, NA, 23, NA, 20.2, 20.2, # 41:50
    20.2, 21.3, NA, 20, 24, 27, 24, 50, 150, 100, # 51:60
    NA, NA, NA, 16
)

usethis::use_data(score_cutoff, compress = "xz", internal = TRUE)
```

---
title: "Genome-wide identification and classification of transcription factors in plant genomes"
author: 
  - name: Fabricio Almeida-Silva
    affiliation: |
      VIB-UGent Center for Plant Systems Biology, Ghent University, 
      Ghent, Belgium
  - name: Yves Van de Peer
    affiliation: |
      VIB-UGent Center for Plant Systems Biology, Ghent University, 
      Ghent, Belgium
output: 
  BiocStyle::html_document:
    self_contained: yes
    toc: true
    toc_depth: 2
    number_sections: yes
date: "`r Sys.Date()`"
bibliography: vignette_bibliography.bib
vignette: >
  %\VignetteIndexEntry{Genome-wide identification and classification of transcription factors in plant genomes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    crop = NULL ## Related to https://stat.ethz.ch/pipermail/bioc-devel/2020-April/016656.html
)
```


# Introduction

Transcription factors (TFs) are proteins that bind to cis-regulatory elements 
in promoter regions of genes and regulate their expression. 
Identifying them in a genome is useful for a variety of reasons, such as 
exploring their evolutionary history across clades and inferring 
gene regulatory networks. `r BiocStyle::Githubpkg("almeidasilvaf/planttfhunter")` 
allows users to identify plant TFs from whole-genome protein sequences and 
classify them into families and subfamilies (when applicable) using the 
classification scheme implemented in [PlantTFDB](http://planttfdb.gao-lab.org/).
As `r BiocStyle::Githubpkg("almeidasilvaf/planttfhunter")` interoperates with 
core Bioconductor packages (i.e., `AAStringSet` objects as input, 
`SummarizedExperiment` objects as output), it can be easily incorporated in 
pipelines for TF identification and classification in large-scale genomic
data sets.


# Installation

You can install `r BiocStyle::Githubpkg("almeidasilvaf/planttfhunter")` 
with the following code:

```{r "install", eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE)) {
      install.packages("BiocManager")
  }

BiocManager::install("planttfhunter")
```

Loading package after installation:

```{r load_package, message = FALSE}
library(planttfhunter)
```

# Data description

In this vignette, we will use protein sequences of TFs from the algae 
species *Galdieria sulphuraria* as an example, as its proteome is
very small. The proteome file was 
downloaded from the PLAZA Diatoms database [@osuna2020seminavis], and 
it was filtered to keep only TFs for demonstration purposes. 
The object `gsu` stores the protein sequences in an `AAStringSet` object.

```{r data}
data(gsu)
gsu
```


# Algorithm description

TF identification and classification is based on the presence of signature 
protein domains, which are identified using profile hidden Markov models (HMMs). 
The family classification scheme is the same as the one used by 
PlantTFDB [@jin2016planttfdb], and it is summarized below: [^1]

```{r scheme, echo = FALSE}
data(classification_scheme)
knitr::kable(classification_scheme)
```


[^1]: **Tip:** You can access this classification scheme in your R session by 
loading the data frame `data(classification_scheme)`.

# Identifying and classifying TFs

To identify TFs from protein sequence data, you will use the 
function `annotate_pfam()`. This function takes as input an `AAStringSet` 
object [^2] and returns a data frame of protein domains associated with 
each sequence. The HMMER program [@finn2011hmmer] is used to scan protein 
sequences for the presence of DNA-binding protein domains, as well as
auxiliary and forbidden domains. Pre-built HMM profiles can be found in the
*extdata/* directory of this package.

[^2]: **Tip:** If you have protein sequences in a FASTA file, you can read 
them into an `AAStringSet` object with the function `readAAStringSet()` from 
the `r BiocStyle::Biocpkg("Biostrings")` package.

This is how you can run `annotate_pfam()` [^3]:

[^3]: **Note:** in the code chunk below, the if statement is not required.
We just added it to make sure that the function `annotate_pfam()` is only
executed if HMMER is installed, to avoid problems when building this
vignette in machines that do not have HMMER installed. 

```{r identifying_tfs}
data(gsu_annotation)

# Annotate TF-related domains using a local installation of HMMER
if(hmmer_is_installed()) {
  gsu_annotation <- annotate_pfam(gsu)
} 

# Take a look at the first few lines of the output
head(gsu_annotation)
```

Now that we have our TF-related domains, we can classify TFs in families 
with the function `classify_tfs()`.

```{r classifying_tfs}
# Classify TFs into families
gsu_families <- classify_tfs(gsu_annotation)

# Take a look at the output
head(gsu_families)

# Count number of TFs per family
table(gsu_families$Family)
```

# Counting TFs per family in multiple species at once

If you want to get TF counts per family for multiple species, you can use
the function `get_tf_counts()`. This function takes a list of `AAStringSet`
objects containing proteomes as input [^4], and it returns a `SummarizedExperiment`
object containing TF counts per family in each species, as well as species
metadata (optional). If you are not familiar with the `SummarizedExperiment`
class, you should consider checking the vignettes of the 
`r BiocStyle::Biocpkg("SummarizedExperiment")` Bioconductor package.

[^4]: **Tip:** If you have whole-genome protein sequences for multiple
species as FASTA files in a given directory, you can read them all as a list
of `AAStringSet` objects with the function `fasta2AAStringSetlist()` from
the Bioconductor package `r BiocStyle::Biocpkg("syntenet")`.

To demonstrate how `get_tf_counts()` works, we will simulate a 
list of 4 `AAStringSet` objects by sampling 50 random genes from the example
data set `gsu` 4 times. 

```{r simulate_data_proteomes}
set.seed(123) # for reproducibility

# Simulate 4 different species by sampling 100 random genes from `gsu`
proteomes <- list(
    Gsu1 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu2 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu3 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu4 = gsu[sample(names(gsu), 50, replace = FALSE)]
)
proteomes
```

Great, we have a list of 4 `AAStringSet` objects. Now, let's also create a 
simulated species metadata data frame for each "species" (simulated).

```{r simulate_data_species_metadata}
# Create simulated species metadata
species_metadata <- data.frame(
    row.names = names(proteomes),
    Division = "Rhodophyta",
    Origin = c("US", "Belgium", "China", "Brazil")
)

species_metadata
```

You can add as many columns as you want to the species metadata data frame, 
but make sure that **species names are in row names**, and 
that `names(proteomes)` match `rownames(species)`, 
otherwise `get_tf_counts()` will return an error. 

Now that we have a list of `AAStringSet` objects and species metadata, we
can execute `get_tf_counts()`. This function uses `annotate_pfam()` under
the hood, so you also need to have HMMER installed and in your PATH to run it.
Here is how you can run it:

```{r get_tf_counts}
data(tf_counts)

# Get TF counts per family in each species as a SummarizedExperiment object
if(hmmer_is_installed()) {
    tf_counts <- get_tf_counts(proteomes, species_metadata)
}

# Take a look at the SummarizedExperiment object
tf_counts

# Look at the matrix of counts: assay() function from SummarizedExperiment
SummarizedExperiment::assay(tf_counts)

# Look at the species metadata: colData() function from SummarizedExperiment
SummarizedExperiment::colData(tf_counts)
```

Cool, huh? In real-world analyses, once you have TF counts per family in 
multiple species obtained with `get_tf_counts()`, 
you can try to find associations between TF counts and
eco-evolutionary aspects or traits of each species (e.g., higher frequencies of
a stress-related TF family in a species that inhabits a stressful environment).

# Session information {.unnumbered}

This document was created under the following conditions:

```{r session_info}
sessioninfo::session_info()
```

# References {.unnumbered}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/planttfhunter-package.R
\docType{package}
\name{planttfhunter-package}
\alias{planttfhunter}
\alias{planttfhunter-package}
\title{planttfhunter: Identification and classification of plant transcription factors}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

planttfhunter is used to identify plant transcription factors (TFs) from protein sequence data and classify them into families and subfamilies using the classification scheme implemented in PlantTFDB. TFs are identified using pre-built hidden Markov model profiles for DNA-binding domains. Then, auxiliary and forbidden domains are used with DNA-binding domains to classify TFs into families and subfamilies (when applicable). Currently, TFs can be classified in 58 different TF families/subfamilies.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/almeidasilvaf/planttfhunter}
  \item Report bugs at \url{https://support.bioconductor.org/t/planttfhunter}
}

}
\author{
\strong{Maintainer}: Fabrício Almeida-Silva \email{fabricio_almeidasilva@hotmail.com} (\href{https://orcid.org/0000-0002-5314-2964}{ORCID})

Authors:
\itemize{
  \item Yves Van de Peer \email{yves.vandepeer@psb.vib-ugent.be} (\href{https://orcid.org/0000-0003-4327-3730}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{hmmer_is_installed}
\alias{hmmer_is_installed}
\title{Check if HMMER is installed}
\usage{
hmmer_is_installed()
}
\value{
Logical indicating whether HMMER is installed or not.
}
\description{
Check if HMMER is installed
}
\examples{
hmmer_is_installed()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_domain_annotation.R
\name{annotate_pfam}
\alias{annotate_pfam}
\title{Annotate proteins sequences with PFAM domains}
\usage{
annotate_pfam(seq = NULL, evalue = 1e-05)
}
\arguments{
\item{seq}{An AAStringSet object as returned
by \code{Biostrings::readAAStringSet()}. The sequences in this object must
represent only the translated sequences of primary (or longest) transcripts.}

\item{evalue}{Numeric indicating the E-value threshold for hmmsearch
to be used for domains without pre-defined domain cutoffs.
Only valid if parameter mode = 'local'. Default: 1e-05.}
}
\value{
A 2-column data frame with the variables \strong{Gene}
and \strong{Domain}, which contain gene IDs and domain IDs, respectively.
}
\description{
PFAM domains are assigned to each sequence using HMMER.
}
\examples{
data(gsu)
seq <- gsu[1:5]
if(hmmer_is_installed()) {
    annotate_pfam(seq)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_tf_counts.R
\name{get_tf_counts}
\alias{get_tf_counts}
\title{Get TF frequencies for each species as a SummarizedExperiment object}
\usage{
get_tf_counts(proteomes, species_metadata = NULL)
}
\arguments{
\item{proteomes}{List of \strong{AAStringSet} objects}

\item{species_metadata}{(Optional) A data frame containing species names
in row names (names must match element names in
the \strong{proteomes} list), and species metadata
(e.g., taxonomic information, ecological information)
in columns. If NULL, the colData of the SummarizedExperiment object
will be empty.}
}
\value{
A SummarizedExperiment object containing transcription factor
frequencies per family in each species, as well as species metadata (if
\strong{species_metadata} is not NULL).
}
\description{
This function identifies and classifies TFs, and returns TF counts for
each family as a SummarizedExperiment object
}
\examples{
data(gsu)

set.seed(123)
# Pick random subsets of 100 genes to simulate other species
proteomes <- list(
    Gsu1 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu2 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu3 = gsu[sample(names(gsu), 50, replace = FALSE)],
    Gsu4 = gsu[sample(names(gsu), 50, replace = FALSE)]
)

# Create species metadata
species_metadata <- data.frame(
    row.names = names(proteomes),
    Division = "Rhodophyta",
    Origin = c("US", "Belgium", "China", "Brazil")
)

# Get SummarizedExperiment object
if(hmmer_is_installed()) {
    se <- get_tf_counts(proteomes, species_metadata)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{classification_scheme}
\alias{classification_scheme}
\title{Data frame of TF family classification scheme}
\format{
A data frame with the following variables:
\describe{
\item{Family}{TF family name.}
\item{Subfamily}{TF subfamily name.}
\item{DBD}{DNA-binding domain}
\item{Auxiliary}{Auxiliary domain}
\item{Forbidden}{Forbidden domain}
}
}
\usage{
data(classification_scheme)
}
\description{
The classification scheme is the same as the one used by PlantTFDB.
}
\examples{
data(classification_scheme)
}
\references{
Jin, J., Tian, F., Yang, D. C., Meng, Y. Q., Kong, L., Luo, J., &
Gao, G. (2016). PlantTFDB 4.0: toward a central hub for transcription
factors and regulatory interactions in plants.
Nucleic acids research, gkw982.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gsu}
\alias{gsu}
\title{Protein sequences of the algae species Galdieria sulphuraria}
\format{
An AAStringSet object as returned
by \code{Biostrings::readAAStringSet()}.
}
\usage{
data(gsu)
}
\description{
Data obtained from PLAZA Diatoms. Only genes containing domains used for
TF family classification were kept for package size issues.
}
\examples{
data(gsu)
}
\references{
Osuna-Cruz, C. M., Bilcke, G., Vancaester, E., De Decker, S., Bones, A. M.,
Winge, P., ... & Vandepoele, K. (2020). The Seminavis robusta genome
provides insights into the evolutionary adaptations of benthic diatoms.
Nature communications, 11(1), 1-13.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tf_counts}
\alias{tf_counts}
\title{TF counts per family in 4 simulated species}
\format{
A SummarizedExperiment with TF frequencies per family in each
species in \strong{assay} and species metadata in \strong{colData}.
}
\usage{
data(tf_counts)
}
\description{
Simulated species were created by sampling 100 genes from the example
data set \strong{gsu} with after \code{set.seed(123)}.
}
\examples{
data(tf_counts)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gsu_families}
\alias{gsu_families}
\title{TFs families of the algae species Galdieria sulphuraria

The data set was created using the funcion \code{classify_tfs()}.}
\format{
A 2-column data frame with the following variables:
\describe{
\item{Gene}{Gene ID}
\item{Family}{TF family}
}
}
\usage{
data(gsu_families)
}
\description{
TFs families of the algae species Galdieria sulphuraria

The data set was created using the funcion \code{classify_tfs()}.
}
\examples{
data(gsu_families)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_classification.R
\name{classify_tfs}
\alias{classify_tfs}
\title{Identify TFs and classify them in families}
\usage{
classify_tfs(domain_annotation = NULL)
}
\arguments{
\item{domain_annotation}{A 2-column data frame with the gene ID in the first
column and the domain ID in the second column.}
}
\value{
A 2-column data frame with the variables \strong{Gene}
and \strong{Family} representing gene ID and TF family, respectively.
}
\description{
Identify TFs and classify them in families
}
\examples{
data(gsu_annotation)
domain_annotation <- gsu_annotation
families <- classify_tfs(domain_annotation)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gsu_annotation}
\alias{gsu_annotation}
\title{Domain annotation for the algae species Galdieria sulphuraria

The data set was created using the funcion \code{annotate_pfam()} in local
mode.}
\format{
A 2-column data frame with the following variables:
\describe{
\item{Gene}{Gene ID}
\item{Annotation}{Domain ID or domain name when ID is not available in PFAM}
}
}
\usage{
data(gsu_annotation)
}
\description{
Domain annotation for the algae species Galdieria sulphuraria

The data set was created using the funcion \code{annotate_pfam()} in local
mode.
}
\examples{
data(gsu_annotation)
}
\keyword{datasets}
